"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolvePathRelativeToConfig = resolvePathRelativeToConfig;
exports.resolveProjectName = resolveProjectName;
exports.resolveSchemaPath = resolveSchemaPath;
exports.getGraphQLProjects = getGraphQLProjects;
exports.getGraphQLSchemaPaths = getGraphQLSchemaPaths;
exports.getGraphQLProjectIncludedFilePaths = getGraphQLProjectIncludedFilePaths;
exports.getGraphQLProjectForSchemaPath = getGraphQLProjectForSchemaPath;
exports.defaultGraphQLProjectName = void 0;

var _fs = require("fs");

var _util = require("util");

var _glob = _interopRequireDefault(require("glob"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// we need to use an import/require here because it does not force consumers to
// enable esModuleInterop in tsconfig.json
const defaultGraphQLProjectName = 'GraphQL'; // temporary utility until `graphql-config` supports this function natively
// see: https://github.com/prisma/graphql-config/pull/113

exports.defaultGraphQLProjectName = defaultGraphQLProjectName;

function resolvePathRelativeToConfig(project, relativePath) {
  return project.resolveConfigPath(relativePath);
}

function resolveProjectName(project, defaultName = defaultGraphQLProjectName) {
  return project.projectName || defaultName;
}

function resolveSchemaPath(project, ignoreMissing = false) {
  // schemaPath is nullable in graphq-config even though it cannot actually be
  // omitted. This function simplifies access to the schemaPath without
  // requiring a type guard.
  if (!project.schemaPath) {
    // this case should never happen with a properly formatted config file.
    // graphql-config currently does not perform any validation so it's possible
    // for a mal-formed schema to be loaded at runtime.
    throw new Error(`Missing GraphQL schemaPath for project '${resolveProjectName(project)}'`);
  } // resolve fully qualified schemaPath


  const schemaPath = project.resolveConfigPath(project.schemaPath);

  if (ignoreMissing) {
    return schemaPath;
  }

  if (!(0, _fs.existsSync)(schemaPath)) {
    const forProject = project.projectName ? ` for project '${project.projectName}'` : '';
    throw new Error([`Schema not found${forProject}.`, `Expected to find the schema at '${schemaPath}' but the path does not exist.`, `Check '${project.configPath}' and verify that schemaPath is configured correctly${forProject}.`].join(' '));
  }

  return schemaPath;
}

function getGraphQLProjects(config) {
  const projects = config.getProjects();

  if (projects) {
    // multi-project configuration, return an array of projects
    return Object.values(projects);
  }

  const project = config.getProjectConfig();

  if (project && project.schemaPath) {
    // single project configuration, return an array of the single project
    return [project];
  } // invalid project configuration


  throw new Error(`No projects defined in '${config.configPath}'`);
}

function getGraphQLSchemaPaths(config) {
  return getGraphQLProjects(config).reduce((schemas, project) => {
    return schemas.concat(resolveSchemaPath(project));
  }, []);
}

async function getGraphQLProjectIncludedFilePaths(projectConfig) {
  return (await Promise.all(projectConfig.includes.map(include => (0, _util.promisify)(_glob.default)(resolvePathRelativeToConfig(projectConfig, include), {
    ignore: projectConfig.excludes.map(exclude => resolvePathRelativeToConfig(projectConfig, exclude))
  })))).reduce((allFilePaths, filePaths) => allFilePaths.concat(filePaths), []);
}

function getGraphQLProjectForSchemaPath(config, schemaPath) {
  const project = getGraphQLProjects(config).filter(project => project.schemaPath === schemaPath).shift() || config.getProjectConfig();

  if (!project || project.schemaPath !== schemaPath) {
    throw new Error(`No project defined in graphql config for schema '${schemaPath}'`);
  }

  return project;
}